#define INCL_WIN
#define INCL_PM

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <os2.h>

#include "Basic.hpp"
#include "Resource.hpp"
#include "Class_DocumentHandling.hpp"

//------------------------------------------------------------------------
// Global variables

typedef struct {
     HWND hmenu;// Handle to the menu
     Class_DocumentHandling *SDH;// Pointer to the (Safe) Document Handling class
} WINDOWDATA, *PWINDOWDATA;

PWINDOWDATA wd;

//------------------------------------------------------------------------
// Function prototypes

MRESULT EXPENTRY Dialog_About( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 );

//------------------------------------------------------------------------
//------------------------------------------------------------------------

MRESULT EXPENTRY Main_Frame( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
     if( msg != WM_CREATE )
     {
          // Retrieve the window word
          wd=(PWINDOWDATA)WinQueryWindowPtr(hwnd, 0);
          if(!wd)
               ErrorBox("Error retrieving window word 0");
     }

     switch(msg)
     {
     //----------------------------------
     case WM_CREATE:
          wd=new WINDOWDATA;
          if(!wd)
          {
               ErrorBox("Error creating a window word in WM_CREATE");
               WinSendMsg(hwnd, WM_CLOSE, 0, 0);
               exit(1);
          }
          if(!WinSetWindowPtr(hwnd, 0, wd)){
               ErrorBox("Error setting window word 0  in WM_CREATE");
               WinSendMsg(hwnd, WM_CLOSE, 0, 0);
               exit(1);
          }
          {
               // Query the handle of the frame window
               HWND hwndFrame = WinQueryWindow( hwnd, QW_PARENT );

               // Query the handle of the menu
               wd->hmenu = WinWindowFromID( hwndFrame, FID_MENU );

               // Create an instance of the (Safe) Document Handling class
               // We pass the handle of the frame window and a macro
               // equal to the string constant defined in Basic.hpp
               wd->SDH = new Class_DocumentHandling( hwndFrame, PRODUCT );
          }
          break;
     //----------------------------------
     case WM_VERIFYSHUTDOWN:
          // Make certain that the application is ready to be closed
          if( wd->SDH->FileNew() )
          {
               // We call IDM_FILENEW so that the current document can
               // be closed and various variables reset
               WinPostMsg( hwnd, WM_COMMAND, MPFROMSHORT( IDM_FILENEW ), 0 );

               // Release allocated memory
               delete wd;

               return (MRESULT) TRUE;// It's OK to shutdown
          }
          else
          {
               // The user selected Cancel, so we stop the application's shutdown
               return (MRESULT) FALSE;// It's not OK to shutdown
          }
     //----------------------------------
     // case WM_CLOSE:
     //     In order to properly handle a shutdown notification,
     //     the code to handle the closure of the application,
     //     such as releasing allocated memory, has been moved
     //     to WM_VERIFYSHUTDOWN (a user message)
     //     break;
     //----------------------------------
     case WM_COMMAND:
          switch( SHORT1FROMMP( mp1 ) )
          {
                //----------------------------------
               case IDM_FILENEW:
                    if( wd->SDH->FileNew() )
                    {
                         // Prepare a new document
                         // Redrawing the screen may be warranted
                    }
                    break;
                //----------------------------------
                case IDM_FILEOPEN:
                    if( wd->SDH->FileOpen() )
                    {
                         // The file has been loaded
                         // Maybe we should display its contents
                    }
                    break;
                //----------------------------------
               case IDM_FILESAVE:
                    if( wd->SDH->FileSave() )
                    {
                         // The file has been saved
                         // Shouldn't require additional code here
                    }
                    break;
                //----------------------------------
               case IDM_FILESAVEAS:
                    if( wd->SDH->FileSaveAs() )
                    {
                         // The file has been saved under the new name
                         // Shouldn't require additional code here
                    }
                    break;
                //----------------------------------
               case IDM_FILECLOSE:
                    // The user selected close from the menu
                    // so we close the application
                    WinSendMsg(hwnd, WM_CLOSE, 0, 0);
                    break;
                //----------------------------------
                case IDM_EDITUNDO:
                    // This flags the current document as modified
                    wd->SDH->SetModified();
                    break;
                //----------------------------------
               case IDM_EDITCOPY:
                    break;
                //----------------------------------
               case IDM_EDITCUT:
                    break;
                //----------------------------------
               case IDM_EDITPASTE:
                    break;
                //----------------------------------
               case IDM_HELPINDEX:
                    break;
                //----------------------------------
               case IDM_HELPGENERAL:
                    break;
                //----------------------------------
               case IDM_HELPUSING:
                    break;
                //----------------------------------
               case IDM_HELPKEYS:
                    break;
                //----------------------------------
                case IDM_HELPABOUT:
                    WinDlgBox(HWND_DESKTOP, hwnd, Dialog_About, 0, IDD_DIALOG_ABOUT, NULL);
                    break;
                //----------------------------------
          }//switch(SHORT1FROMMP(mp1))
          break;
     //----------------------------------
     case WM_PAINT:
     {
          HPS hps=WinBeginPaint(hwnd, NULLHANDLE, NULLHANDLE);
          RECTL rcl;
          WinQueryWindowRect(hwnd, &rcl);
          WinFillRect(hps, &rcl, CLR_BACKGROUND);
          WinEndPaint(hps);
     }
     return 0;

     //----------------------------------
     }//switch(msg)

     return WinDefWindowProc(hwnd, msg, mp1, mp2);

}//Basic Frame procedure

